@@ -1,12 +1,12 @@
 package HTML::Template::Compiled::Plugin::InlineImage;
-# $Id: InlineImage.pm,v 1.10 2006/08/27 10:30:44 tinita Exp $
+# $Id: InlineImage.pm,v 1.14 2006/09/14 10:28:35 tinita Exp $
 use strict;
 use warnings;
 use Carp qw(croak carp);
 use HTML::Template::Compiled::Expression qw(:expressions);
 use HTML::Template::Compiled;
 use MIME::Base64;
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 HTML::Template::Compiled->register(__PACKAGE__);
 our $SIZE_WARNING = 1;
 
@@ -49,9 +49,22 @@ sub inline {
     my (%args) = @_;
     my $image = $args{image};
     my $type = $args{type};
-    my ($binary, $width, $height) = ref $image eq 'GD::Image'
-        ? gd_to_binary($image,$type)
-        : croak "unknown image type " . ref $image;
+    my ($binary, $width, $height);
+    unless (ref $image) {
+        # we have raw data, try guessing mime type
+        require File::MMagic;
+        my $mm = File::MMagic->new;
+        my $mtype = $mm->checktype_contents($image)
+            or croak "Could not determine mime type";
+        my ($type_a,$type_b) = split m#/#, $mtype;
+        $type = $type_b;
+        $binary = $image;
+    }
+    else {
+        ($binary, $width, $height) = ref $image eq 'GD::Image'
+            ? gd_to_binary($image,$type)
+            : croak "unknown image type " . ref $image;
+    }
     my $base64 = encode_base64($binary);
     my $string = "data:image/$type;base64,$base64";
     my $l = length $string;
@@ -95,6 +108,7 @@ The Perl code:
         tagstyle => [qw(+tt)],
     );
     $htc->param(gd_object => $gd);
+    $htc->param(raw_data => $data_from_image_file);
     print $htc->output;
 
 The Template:
@@ -102,11 +116,12 @@ The Template:
     <html>
         <body>
         <img [%= gd_object escape="INLINE_IMG" %] alt="[Rendered GD Image]">
+        <img [%= raw_data escape="INLINE_IMG" %] alt="[Rendered Image]">
         </body>
     </html>
 
-This will create an inline image. The GD-object output is turned into base64
-and put into a src attribute.
+This will create an inline image. The GD-object/the raw image output
+is turned into base64 and put into a src attribute.
 
 The output looks like
 
@@ -121,8 +136,8 @@ C<$HTML::Template::Compiled::Plugin::InlineImage::SIZE_WARNING> to 0.
 =head1 DESCRIPTION
 
 This is a plugin for L<HTML::Template::Compiled>. If you feed it GD-objects
-(other image-object-types could be added in the future), then it
-will render the object like described in RFC 2397
+or raw images (other image-object-types could be added in the future),
+then it will render the object like described in RFC 2397
 (http://www.ietf.org/rfc/rfc2397.txt).
 
 =head1 ESCAPE TYPES
@@ -145,7 +160,8 @@ renders as jpeg
 
 =item INLINE_IMG
 
-renders as png
+renders per default as png, or if you feed it raw data, it will guess
+the image type.
 
 =back
 
@@ -194,11 +210,23 @@ This function is usually only used internally.
 
 =back
 
-=cut
+=head1 ERROR MESSAGES, WARNINGS
+
+If your image is (in base64, plus mime type) bigger than 1024 bytes,
+you'll get a warning like:
+
+    Image is too big (1234 characters > 1024)
+
+To avoid the warning, set
+C<$HTML::Template::Compiled::Plugin::InlineImage::SIZE_WARNING> to 0.
 
 EXAMPLES
 
-See L<"examples/gd.pl"> and L<"examples/gd.html"> for a simple example.
+Simple examples:
+
+L<"examples/gd.pl"> and L<"examples/gd.html">
+
+L<"examples/raw.pl"> and L<"examples/raw.html">
 
 =head1 SEE ALSO
 
@@ -212,3 +240,4 @@ This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.5 or,
 at your option, any later version of Perl 5 you may have available.
 
+=cut
